﻿using System;
using CellularAutomata.RuleSets;

// Written by Kory Becker
// PrimaryObjects
// http://www.primaryobjects.com
namespace CellularAutomata
{
    class Program
    {
        private const char cellIcon = 'o';
        private const int maxX = 60;
        private const int maxY = 40;
        private static int[,] field = new int[maxX, maxY];

        static void DrawField()
        {
            Console.CursorLeft = 0;
            Console.CursorTop = 0;

            for (int y = 0; y < maxY; y++)
            {
                for (int x = 0; x < maxX; x++)
                {
                    Console.Write(field[x, y] == 1 ? cellIcon : ' ');
                }

                Console.WriteLine();
            }
        }

        static void Main(string[] args)
        {
            // Initialize
            int startX = maxX / 2;
            int startY = maxY / 2;

            Console.SetWindowSize(maxX + 10, maxY + 10);
            Console.SetWindowPosition(0, 0);

            #region Initial Cell Formations

            /*field[maxSize / 2, maxSize / 2 - 1] = 1;
            field[maxSize / 2 + 1, maxSize / 2] = 1;
            field[maxSize / 2 - 1, maxSize / 2 + 1] = 1;
            field[maxSize / 2, maxSize / 2 + 1] = 1;
            field[maxSize / 2 + 1, maxSize / 2 + 1] = 1;*/

            /*// 2x2 block - crawls across screen when used with RuleSet Gliders().
            field[startX, startY] = 1;
            field[startX + 1, startY] = 1;
            field[startX, startY + 1] = 1;
            field[startX + 1, startY + 1] = 1;*/

            /*// Glider - use with RuleSet ConwaysGameOfLife().
            field[startX + 1, startY + 2] = 1;
            field[startX + 2, startY + 3] = 1;
            field[startX + 3, startY + 1] = 1;
            field[startX + 3, startY + 2] = 1;
            field[startX + 3, startY + 3] = 1;*/

            /*// Exploder - use with RuleSet ConwaysGameOfLife().
            field[startX - 2, startY] = 1;
            field[startX - 2, startY + 1] = 1;
            field[startX - 2, startY + 2] = 1;
            field[startX - 2, startY + 3] = 1;
            field[startX - 2, startY + 4] = 1;
            field[startX, startY] = 1;
            field[startX, startY + 4] = 1;
            field[startX + 2, startY] = 1;
            field[startX + 2, startY + 1] = 1;
            field[startX + 2, startY + 2] = 1;
            field[startX + 2, startY + 3] = 1;
            field[startX + 2, startY + 4] = 1;*/

            /*// R-Pentamino
            field[startX - 1, startY + 1] = 1;
            field[startX, startY] = 1;
            field[startX, startY + 1] = 1;
            field[startX, startY + 2] = 1;
            field[startX + 1, startY] = 1;*/

            /*// Random
            Random r = new Random((int)DateTime.Now.Ticks);
            for (int x = 0; x < maxX; x++)
            {
                for (int y = 0; y < maxY; y++)
                {
                    field[x, y] = r.Next(0, 1 + 1);
                }
            }*/

            #endregion

            // Cross - becomes stable - use with RuleSet ConwaysGameOfLife().
            for (int x = 0; x < maxX; x++)
            {
                field[x, startY] = 1;
            }
            for (int y = 0; y < maxY; y++)
            {
                field[startX, y] = 1;
            }

            // Instantiate the desired concrete RuleSet in this Strategy pattern. For example, ConwaysGameOfLife(field, maxX, maxY) or RuleGeneric(field, maxX, maxY, 47, 3), Rule1358d357(), etc.
            RuleSet ruleSet = new ConwaysGameOfLife(field, maxX, maxY);

            for (int i = 0; i < 5000; i++)
            {
                DrawField();
                ruleSet.Tick();
            }
        }
    }
}
